..\build\Windows\binaries\bin\Release\parameterize FLOAT_ELL DATA/girl_face_hires3.obj out.obj
pause
REM un comment the two following lines if you have a 1.3 CUDA device
REM ..\build\Windows\binaries\bin\Release\parameterize DOUBLE_ELL DATA/girl_face_hires3.obj out.obj
REM pause
REM Example with a MatrixMarket file
REM ..\build\Windows\binaries\bin\Release\mmtx CG 10000 1e-6 DATA/5pt_10x10.mtx
REM pause
